package com.example.karro.locationapp;

import android.Manifest;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Build;
import android.provider.Settings;
import android.support.v4.app.FragmentActivity;
import android.os.Bundle;
import android.util.Log;
import android.content.DialogInterface;
import android.view.View;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;

import java.security.Permission;
import java.util.ArrayList;
import java.util.HashSet;
import java.text.DecimalFormat;
import java.util.HashMap;

public class MapsActivity extends FragmentActivity implements OnMapReadyCallback, LocationListener {

    final static int PERMISSION_ALL = 1;
    final static String[] PERMISSIONS = {Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_FINE_LOCATION};

    private GoogleMap mMap;

    private static MarkerOptions mo_user;
    private static MarkerOptions mo_carelia;
    private static MarkerOptions mo_louhi;
    private static MarkerOptions mo_tori36;
    private static MarkerOptions mo_aura;
    private static MarkerOptions mo_futura;

    private Marker marker_user;
    private Marker marker_carelia;
    private Marker marker_louhi;
    private Marker marker_tori36;
    private Marker marker_aura;
    private Marker marker_futura;

    private LocationManager lm;
    private Location location;

    private static boolean nearest_carelia = false;
    private static boolean nearest_louhi = false;
    private static boolean nearest_tori36 = false;
    private static boolean nearest_aura = false;
    private static boolean nearest_futura = false;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_maps);
        // Obtain the SupportMapFragment and get notified when the map is ready to be used.
        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);

        lm = (LocationManager)getSystemService(LOCATION_SERVICE);

        if (Build.VERSION.SDK_INT >= 23 && !isPermissionGranted()){
            requestPermissions(PERMISSIONS, PERMISSION_ALL);
        }
        else{
            requestLocation();
        }
        if (!isLocationEnabled()){
            showAlert(1);
        }
    }


    /**
     * Manipulates the map once available.
     * This callback is triggered when the map is ready to be used.
     * This is where we can add markers or lines, add listeners or move the camera. In this case,
     * we just add a marker near Sydney, Australia.
     * If Google Play services is not installed on the device, the user will be prompted to install
     * it inside the SupportMapFragment. This method will only be triggered once the user has
     * installed Google Play services and returned to the app.
     */
    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;

        mo_user = new MarkerOptions().position(new LatLng(0, 0)).title("Your current location");

        //mo_user = mo_user.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_YELLOW));
        mo_user = mo_user.icon(BitmapDescriptorFactory.fromResource(R.drawable.custom_marker1));
        marker_user = mMap.addMarker(mo_user);

        mo_carelia = new MarkerOptions().position(new LatLng(62.603927, 29.743756)).title("Carelia");
        mo_louhi = new MarkerOptions().position(new LatLng(62.597943, 29.744132)).title("Louhi");
        mo_tori36 = new MarkerOptions().position(new LatLng(62.605618, 29.767853)).title("Tori36");
        mo_aura = new MarkerOptions().position(new LatLng(62.603749, 29.746208)).title("Aura");
        mo_futura = new MarkerOptions().position(new LatLng(62.604999, 29.741046)).title("Futura");

        updateNearest();

        mo_user.position(new LatLng(requestLocation().getLatitude(), requestLocation().getLongitude()));

        mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(mo_user.getPosition(),15));
        // Zoom in, animating the camera.
        mMap.animateCamera(CameraUpdateFactory.zoomIn());
        // Zoom out to zoom level 10, animating with a duration of 2 seconds.
        mMap.animateCamera(CameraUpdateFactory.zoomTo(15), 2000, null);


    }

    public void updateNearest() {
        getNearest(requestLocation().getLatitude(), requestLocation().getLongitude());
        mo_user.position(new LatLng(location.getLatitude(), location.getLongitude()));
        if (nearest_carelia) {
            mo_carelia = mo_carelia.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_BLUE));
            marker_carelia = mMap.addMarker(mo_carelia);
            nearest_carelia = false;
        } else {
            mo_carelia = mo_carelia.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN));
            marker_carelia = mMap.addMarker(mo_carelia);
        }

        if (nearest_louhi) {
            mo_louhi = mo_louhi.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_BLUE));
            marker_louhi = mMap.addMarker(mo_louhi);
            nearest_louhi = false;
        } else {
            mo_louhi = mo_louhi.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN));
            marker_louhi = mMap.addMarker(mo_louhi);
        }

        if (nearest_tori36) {
            mo_tori36 = mo_tori36.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_BLUE));
            marker_tori36 = mMap.addMarker(mo_tori36);
            nearest_tori36 = false;
        } else {
            mo_tori36 = mo_tori36.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN));
            marker_tori36 = mMap.addMarker(mo_tori36);
        }

        if (nearest_aura) {
            mo_aura = mo_aura.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_BLUE));
            marker_aura = mMap.addMarker(mo_aura);
            nearest_aura = false;
        } else {
            mo_aura = mo_aura.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN));
            marker_aura = mMap.addMarker(mo_aura);
        }

        if (nearest_futura) {
            mo_futura = mo_futura.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_BLUE));
            marker_futura = mMap.addMarker(mo_futura);
            nearest_futura = false;
        } else {
            mo_futura = mo_futura.icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN));
            marker_futura = mMap.addMarker(mo_futura);
        }
    }

    @Override
    public void onLocationChanged(Location location) {
        LatLng myCoordinates = new LatLng(location.getLatitude(), location.getLongitude());
        marker_user.setPosition(myCoordinates);
        updateNearest();
        mMap.moveCamera(CameraUpdateFactory.newLatLng(myCoordinates));
    }

    private boolean isPermissionGranted(){
        if (checkSelfPermission(Manifest.permission.ACCESS_COARSE_LOCATION)
            == PackageManager.PERMISSION_GRANTED || checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION)
                == PackageManager.PERMISSION_GRANTED) {
            Log.v("myLog", "Permission is granted");
            return true;
        }
        else {
            Log.v("myLog", "Permission is not granted");
            return false;
        }
    }

    private Location requestLocation(){
        Criteria criteria = new Criteria();
        criteria.setAccuracy(Criteria.ACCURACY_FINE);
        criteria.setPowerRequirement(Criteria.POWER_HIGH);
        String provider = lm.getBestProvider(criteria, true);
        lm.requestLocationUpdates(provider, 10000, 10, this);
        location = lm.getLastKnownLocation(LocationManager.GPS_PROVIDER);
        return location;

    }

    private boolean isLocationEnabled() {
        return (lm.isProviderEnabled(LocationManager.GPS_PROVIDER) ||
                lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER));
    }

    private void showAlert(final int status) {
        String message, title, btnText;
        if (status == 1) {
            message = "Your locations settings are set 'Off'.\nPlease enable location to use this app.";
            title = "Enable location";
            btnText = "Location Settings";
        } else {
            message = "Please allow this app to access location!";
            title = "Permission access";
            btnText = "Grant";
        }
        final AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        dialog.setCancelable(false);
        dialog.setTitle(title)
                .setMessage(message)
                .setPositiveButton(btnText, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface paramDialogInterface, int paramInt) {
                        if (status == 1) {
                            Intent myIntent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                            startActivity(myIntent);
                        } else
                            requestPermissions(PERMISSIONS, PERMISSION_ALL);
                    }
                })
                .setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface paramDialogInterface, int paramInt) {
                        finish();
                    }
                });
        dialog.show();

    }

    @Override
    public void onProviderEnabled(String s) {

    }

    @Override
    public void onProviderDisabled(String s) {

    }

    @Override
    public void onStatusChanged(String s, int i, Bundle bundle) {

    }

    public static float getDistance(double lat1, double lat2, double lng1, double lng2) {
        float[] results = new float[1];
        Location.distanceBetween(lat1, lng1, lat2, lng2, results);
        return results[0];

    }

    private void getNearest(double lat, double lng) {

        float dist_carelia = getDistance(mo_carelia.getPosition().latitude, lat, mo_carelia.getPosition().longitude, lng);
        float dist_louhi = getDistance(mo_louhi.getPosition().latitude, lat, mo_louhi.getPosition().longitude, lng);
        float dist_tori36 = getDistance(mo_tori36.getPosition().latitude, lat, mo_tori36.getPosition().longitude, lng);
        float dist_aura = getDistance(mo_aura.getPosition().latitude, lat, mo_aura.getPosition().longitude, lng);
        float dist_futura = getDistance(mo_futura.getPosition().latitude, lat, mo_futura.getPosition().longitude, lng);

        double d_carelia = (double) dist_carelia;
        double d_louhi = (double) dist_louhi;
        double d_tori36 = (double) dist_tori36;
        double d_aura = (double) dist_aura;
        double d_futura = (double) dist_futura;

        HashSet<Double> set = new HashSet<Double>();

        set.add(d_carelia);
        set.add(d_louhi);
        set.add(d_tori36);
        set.add(d_aura);
        set.add(d_futura);

        double min = -1;

        for (Double d : set){
            //System.out.println("\ndistance: " + d);
            if (d < min || min == -1) {
                min  = d;
            }
        }

        if (min == dist_carelia) {
            //System.out.println("Carelia lähin");
            nearest_carelia = true;
        }
        if (min == dist_louhi) {
            //System.out.println("Louhi lähin");
            nearest_louhi = true;
        }
        if (min == dist_tori36) {
            //System.out.println("Tori36 lähin");
            nearest_tori36 = true;
        }
        if (min == dist_aura) {
            //System.out.println("Aura lähin");
            nearest_aura = true;
        }
        if (min == dist_futura) {
            //System.out.println("Futura lähin");
            nearest_futura = true;
        }
    }

    public void moveToListView(View view) {
        Intent intent = new Intent(MapsActivity.this, ListActivity.class);
        MapsActivity.this.startActivity(intent);
    }

    public static HashMap<String, LatLng> getLatLng() {
        HashMap<String, LatLng> map = new HashMap<String, LatLng>();
        map.put("User", mo_user.getPosition());
        map.put("Carelia", mo_carelia.getPosition());
        map.put("Louhi", mo_louhi.getPosition());
        map.put("Tori36", mo_tori36.getPosition());
        map.put("Aura", mo_aura.getPosition());
        map.put("Futura", mo_futura.getPosition());
        return map;
    }

    public static ArrayList<String> getNames() {
        ArrayList<String> list = new ArrayList<String>();
        list.add("Carelia");
        list.add("Louhi");
        list.add("Tori36");
        list.add("Aura");
        list.add("Futura");
        return list;
    }

}
